using Posechaemost.Data.LocalData.Entity;
using Posechaemost.Data.Repository;
using Posechaemost.Domain.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Posechaemost.Domain.UseCase
{
    public class UserUseCase
    {
        
        private readonly UserRepositoryImpl _repositoryUserImpl;
        private readonly IGroupRepository _repositoryGroupImpl;

        public UserUseCase(UserRepositoryImpl repositoryImpl, GroupRepositoryImpl repositoryGroupImpl)
        {
            _repositoryUserImpl = repositoryImpl;
            _repositoryGroupImpl = repositoryGroupImpl;
        }

      private List<Group> GetAllGroups() => _repositoryGroupImpl.GetAllGroups()
            .Select(it => new Group { Id  = it.Id, Name = it.Name}).ToList();
      public List<User> GetAllUsers() => _repositoryUserImpl.GetAllUsers
            .Join(_repositoryGroupImpl.GetAllGroups(),
            user => user.GroupID,
            group => group.Id,
            (user, group) => 
            new User { FIO = user.FIO, 
                Guid = user.Guid, 
                GroupId = new Group {Id = group.Id, Name = group.Name } }
            ).ToList();
            

        public bool RemoveUserByGuid(Guid userGuid) {
           return _repositoryUserImpl.RemoveUserByGuid(userGuid);
        }
        public User UpdateUser(User user) {
            UserLocalEntity userLocalEnity = new UserLocalEntity { FIO = user.FIO, GroupID = user.GroupId.Id, Guid = user.Guid };
            UserLocalEntity? result = _repositoryUserImpl.UpdateUser(userLocalEnity);
            if (result == null) throw new Exception("");
            Group? group = GetAllGroups().FirstOrDefault(it => it.Id == result!.GroupID);
            if (group == null) throw new Exception("");
            return new User { FIO = user.FIO, Guid = user.Guid,  GroupId = group};
        }

        public UserLocalEntity GetUserByGuid(Guid userGuid) {
            return _repositoryUserImpl.GetUserByGuid(userGuid);
        }

        public User UpdateUserByGuid(Guid userGuid, String fio, String groupId) {
            UserLocalEntity? result = _repositoryUserImpl.UpdateUserByGuid(userGuid);
            if (result == null) throw new Exception("");
            Group? group = GetAllGroups().FirstOrDefault(it => it.Id == int.Parse(groupId));
            if (group == null) throw new Exception("");
            return new User { FIO = fio, GroupId = group, Guid = userGuid };
        }
    }
}